@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <time.h>
 
 #include "gdkcolor.h"
@@ -31,6 +32,7 @@
 #include "gdkx.h"
 #include "gdkprivate-x11.h"
 #include "gdkscreen-x11.h"
+#include "gdkalias.h"
 
 typedef struct _GdkColormapPrivateX11  GdkColormapPrivateX11;
 
@@ -38,7 +40,6 @@ struct _GdkColormapPrivateX11
 {
   GdkScreen *screen;
   Colormap xcolormap;
-  Display *xdisplay;
   gint private_val;
 
   GHashTable *hash;
@@ -143,6 +144,7 @@ gdk_colormap_finalize (GObject *object)
   
   g_free (private->info);
   g_free (colormap->colors);
+  g_free (private);
   
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -174,7 +176,7 @@ gdk_colormap_new (GdkVisual *visual,
    * kind of default construction (and construct-only arguments)
    */
   
-  g_return_val_if_fail (visual != NULL, NULL);
+  g_return_val_if_fail (GDK_IS_VISUAL (visual), NULL);
 
   colormap = g_object_new (GDK_TYPE_COLORMAP, NULL);
   private = GDK_COLORMAP_PRIVATE_DATA (colormap);
@@ -204,18 +206,23 @@ gdk_colormap_new (GdkVisual *visual,
 
       if (allocate)
 	{
+          GdkVisual *system_visual;
 	  XColor *default_colors;
+          gint n_default_colors;
+
+          system_visual = gdk_screen_get_system_visual (private->screen);
+	  n_default_colors = MIN (system_visual->colormap_size, colormap->size);
 
  	  default_colors = g_new (XColor, colormap->size);
 
-	  for (i = 0; i < colormap->size; i++)
+	  for (i = 0; i < n_default_colors; i++)
 	    default_colors[i].pixel = i;
 	  
 	  XQueryColors (xdisplay,
 			DefaultColormapOfScreen (GDK_SCREEN_X11 (private->screen)->xscreen),
-			default_colors, colormap->size);
+			default_colors, n_default_colors);
 
-	  for (i = 0; i < colormap->size; i++)
+	  for (i = 0; i < n_default_colors; i++)
 	    {
 	      colormap->colors[i].pixel = default_colors[i].pixel;
 	      colormap->colors[i].red = default_colors[i].red;
@@ -223,7 +230,7 @@ gdk_colormap_new (GdkVisual *visual,
 	      colormap->colors[i].blue = default_colors[i].blue;
 	    }
 
-	  gdk_colormap_change (colormap, colormap->size);
+	  gdk_colormap_change (colormap, n_default_colors);
 	  
 	  g_free (default_colors);
 	}
@@ -375,6 +382,8 @@ gdk_colormap_sync (GdkColormap *colormap,
  * Gets the system's default colormap for @screen
  *
  * Returns: the default colormap for @screen.
+ *
+ * Since: 2.2
  */
 GdkColormap *
 gdk_screen_get_system_colormap (GdkScreen *screen)
@@ -560,7 +569,7 @@ gdk_colormap_change (GdkColormap *colormap,
  * For full documentation of the fields, see 
  * the Xlib documentation for <function>XAllocColorCells()</function>.
  * 
- * Return value: 
+ * Return value: %TRUE if the allocation was successful
  **/
 gboolean
 gdk_colors_alloc (GdkColormap   *colormap,
@@ -574,7 +583,7 @@ gdk_colors_alloc (GdkColormap   *colormap,
   gint return_val;
   gint i;
 
-  g_return_val_if_fail (GDK_IS_COLORMAP (colormap), 0);
+  g_return_val_if_fail (GDK_IS_COLORMAP (colormap), FALSE);
 
   private = GDK_COLORMAP_PRIVATE_DATA (colormap);
 
@@ -586,7 +595,7 @@ gdk_colors_alloc (GdkColormap   *colormap,
 				 nplanes, pixels, npixels);
   if (return_val)
     {
-      for (i=0; i<npixels; i++)
+      for (i = 0; i < npixels; i++)
 	{
 	  private->info[pixels[i]].ref_count++;
 	  private->info[pixels[i]].flags |= GDK_COLOR_WRITEABLE;
@@ -633,7 +642,7 @@ gdk_colors_free (GdkColormap *colormap,
   
   pixels_to_free = g_new (gulong, npixels);
 
-  for (i=0; i<npixels; i++)
+  for (i = 0; i < npixels; i++)
     {
       gulong pixel = pixels[i];
       
@@ -651,7 +660,7 @@ gdk_colors_free (GdkColormap *colormap,
 	}
     }
 
-  if (npixels_to_free && !private->screen->closed)
+  if (npixels_to_free && !private->private_val && !private->screen->closed)
     XFreeColors (GDK_SCREEN_XDISPLAY (private->screen), private->xcolormap,
 		 pixels_to_free, npixels_to_free, planes);
   g_free (pixels_to_free);
@@ -690,7 +699,7 @@ gdk_colormap_free_colors (GdkColormap *colormap,
 
   pixels = g_new (gulong, ncolors);
 
-  for (i=0; i<ncolors; i++)
+  for (i = 0; i < ncolors; i++)
     {
       gulong pixel = colors[i].pixel;
       
@@ -708,7 +717,7 @@ gdk_colormap_free_colors (GdkColormap *colormap,
 	}
     }
 
-  if (npixels && !private->screen->closed)
+  if (npixels && !private->private_val && !private->screen->closed)
     XFreeColors (GDK_SCREEN_XDISPLAY (private->screen), private->xcolormap,
 		 pixels, npixels, 0);
 
@@ -751,6 +760,7 @@ gdk_colormap_alloc1 (GdkColormap *colormap,
 	    {
 	      XFreeColors (GDK_SCREEN_XDISPLAY (private->screen), private->xcolormap,
 			   &xcolor.pixel, 1, 0);
+	      private->info[ret->pixel].ref_count++;
 	    }
 	  else
 	    {
@@ -789,7 +799,7 @@ gdk_colormap_alloc_colors_writeable (GdkColormap *colormap,
   if (private->private_val)
     {
       index = 0;
-      for (i=0; i<ncolors; i++)
+      for (i = 0; i < ncolors; i++)
 	{
 	  while ((index < colormap->size) && (private->info[index].ref_count != 0))
 	    index++;
@@ -804,7 +814,7 @@ gdk_colormap_alloc_colors_writeable (GdkColormap *colormap,
 	  else
 	    break;
 	}
-      return i;
+      return ncolors - i;
     }
   else
     {
@@ -815,9 +825,10 @@ gdk_colormap_alloc_colors_writeable (GdkColormap *colormap,
 				  FALSE, NULL, 0, pixels, ncolors);
       if (status)
 	{
-	  for (i=0; i<ncolors; i++)
+	  for (i = 0; i < ncolors; i++)
 	    {
 	      colors[i].pixel = pixels[i];
+	      success[i] = TRUE;
 	      private->info[pixels[i]].ref_count++;
 	      private->info[pixels[i]].flags |= GDK_COLOR_WRITEABLE;
 	    }
@@ -825,7 +836,7 @@ gdk_colormap_alloc_colors_writeable (GdkColormap *colormap,
       
       g_free (pixels);
 
-      return status ? ncolors : 0; 
+      return status ? 0 : ncolors; 
     }
 }
 
@@ -844,12 +855,11 @@ gdk_colormap_alloc_colors_private (GdkColormap *colormap,
   gint nremaining = 0;
   
   private = GDK_COLORMAP_PRIVATE_DATA (colormap);
-  index = -1;
 
   /* First, store the colors we have room for */
 
   index = 0;
-  for (i=0; i<ncolors; i++)
+  for (i = 0; i < ncolors; i++)
     {
       if (!success[i])
 	{
@@ -862,12 +872,18 @@ gdk_colormap_alloc_colors_private (GdkColormap *colormap,
 	      store[nstore].blue = colors[i].blue;
 	      store[nstore].green = colors[i].green;
 	      store[nstore].pixel = index;
+	      store[nstore].flags = DoRed | DoGreen | DoBlue;
 	      nstore++;
 
 	      success[i] = TRUE;
-
 	      colors[i].pixel = index;
+
+	      colormap->colors[index] = colors[i];
 	      private->info[index].ref_count++;
+
+	      g_hash_table_insert (private->hash,
+				   &colormap->colors[index],
+				   &colormap->colors[index]);
 	    }
 	  else
 	    nremaining++;
@@ -884,9 +900,9 @@ gdk_colormap_alloc_colors_private (GdkColormap *colormap,
 
       gchar *available = g_new (gchar, colormap->size);
       for (i = 0; i < colormap->size; i++)
-	available[i] = TRUE;
+	available[i] = !(private->info[i].flags & GDK_COLOR_WRITEABLE);
 
-      for (i=0; i<ncolors; i++)
+      for (i = 0; i < ncolors; i++)
 	{
 	  if (!success[i])
 	    {
@@ -906,7 +922,7 @@ gdk_colormap_alloc_colors_private (GdkColormap *colormap,
       g_free (available);
     }
 
-  return (ncolors - nremaining);
+  return nremaining;
 }
 
 static gint
@@ -923,9 +939,8 @@ gdk_colormap_alloc_colors_shared (GdkColormap *colormap,
   gint nfailed = 0;
 
   private = GDK_COLORMAP_PRIVATE_DATA (colormap);
-  index = -1;
 
-  for (i=0; i<ncolors; i++)
+  for (i = 0; i < ncolors; i++)
     {
       if (!success[i])
 	{
@@ -947,7 +962,7 @@ gdk_colormap_alloc_colors_shared (GdkColormap *colormap,
       
       while (nremaining > 0)
 	{
-	  for (i=0; i<ncolors; i++)
+	  for (i = 0; i < ncolors; i++)
 	    {
 	      if (!success[i])
 		{
@@ -992,13 +1007,13 @@ gdk_colormap_alloc_colors_shared (GdkColormap *colormap,
   /* Change back the values we flagged as permanent failures */
   if (nfailed > 0)
     {
-      for (i=0; i<ncolors; i++)
+      for (i = 0; i < ncolors; i++)
 	if (success[i] == 2)
 	  success[i] = FALSE;
       nremaining = nfailed;
     }
   
-  return (ncolors - nremaining);
+  return nremaining;
 }
 
 static gint
@@ -1018,7 +1033,7 @@ gdk_colormap_alloc_colors_pseudocolor (GdkColormap *colormap,
 
   /* Check for an exact match among previously allocated colors */
 
-  for (i=0; i<ncolors; i++)
+  for (i = 0; i < ncolors; i++)
     {
       if (!success[i])
 	{
@@ -1061,37 +1076,17 @@ gdk_colormap_alloc_colors_pseudocolor (GdkColormap *colormap,
  *    existing colors if the colors cannot be allocated as requested.
  * @success: An array of length @ncolors. On return, this
  *   indicates whether the corresponding color in @colors was
- *   sucessfully allocated or not.
+ *   successfully allocated or not.
  * 
  * Allocates colors from a colormap.
  * 
- * Return value: The number of colors that were not sucessfully 
+ * Return value: The number of colors that were not successfully 
  * allocated.
  **/
 gint
 gdk_colormap_alloc_colors (GdkColormap *colormap,
 			   GdkColor    *colors,
 			   gint         ncolors,
-/**
- * gdk_colormap_alloc_colors:
- * @colormap: a #GdkColormap.
- * @colors: The color values to allocate. On return, the pixel
- *    values for allocated colors will be filled in.
- * @ncolors: The number of colors in @colors.
- * @writeable: If %TRUE, the colors are allocated writeable
- *    (their values can later be changed using gdk_color_change()).
- *    Writeable colors cannot be shared between applications.
- * @best_match: If %TRUE, GDK will attempt to do matching against
- *    existing colors if the colors cannot be allocated as requested.
- * @success: An array of length @ncolors. On return, this
- *   indicates whether the corresponding color in @colors was
- *   sucessfully allocated or not.
- * 
- * Allocates colors from a colormap.
- * 
- * Return value: The number of colors that were not sucessfully 
- * allocated.
- **/
 			   gboolean     writeable,
 			   gboolean     best_match,
 			   gboolean    *success)
@@ -1104,16 +1099,15 @@ gdk_colormap_alloc_colors (GdkColormap *colormap,
 
   g_return_val_if_fail (GDK_IS_COLORMAP (colormap), ncolors);
   g_return_val_if_fail (colors != NULL, ncolors);
+  g_return_val_if_fail (success != NULL, ncolors);
 
   private = GDK_COLORMAP_PRIVATE_DATA (colormap);
 
   if (private->screen->closed)
     return ncolors;
 
-  for (i=0; i<ncolors; i++)
-    {
-      success[i] = FALSE;
-    }
+  for (i = 0; i < ncolors; i++)
+    success[i] = FALSE;
 
   switch (colormap->visual->type)
     {
@@ -1131,9 +1125,23 @@ gdk_colormap_alloc_colors (GdkColormap *colormap,
     case GDK_VISUAL_TRUE_COLOR:
       visual = colormap->visual;
 
-      for (i=0; i<ncolors; i++)
+      for (i = 0; i < ncolors; i++)
 	{
-	  colors[i].pixel = (((colors[i].red >> (16 - visual->red_prec)) << visual->red_shift) +
+	  /* If bits not used for color are used for something other than padding,
+	   * it's likely alpha, so we set them to 1s.
+	   */
+	  guint padding, unused;
+
+	  /* Shifting by >= width-of-type isn't defined in C */
+	  if (visual->depth >= 32)
+	    padding = 0;
+	  else
+	    padding = ((~(guint32)0)) << visual->depth;
+	  
+	  unused = ~ (visual->red_mask | visual->green_mask | visual->blue_mask | padding);
+	  
+	  colors[i].pixel = (unused +
+			     ((colors[i].red >> (16 - visual->red_prec)) << visual->red_shift) +
 			     ((colors[i].green >> (16 - visual->green_prec)) << visual->green_shift) +
 			     ((colors[i].blue >> (16 - visual->blue_prec)) << visual->blue_shift));
 	  success[i] = TRUE;
@@ -1141,7 +1149,7 @@ gdk_colormap_alloc_colors (GdkColormap *colormap,
       break;
     case GDK_VISUAL_STATIC_GRAY:
     case GDK_VISUAL_STATIC_COLOR:
-      for (i=0; i<ncolors; i++)
+      for (i = 0; i < ncolors; i++)
 	{
 	  xcolor.red = colors[i].red;
 	  xcolor.green = colors[i].green;
@@ -1282,6 +1290,8 @@ gdk_color_change (GdkColormap *colormap,
  *   with gdk_x11_colormap_foreign_new(), unref'ing the last
  *   reference to the object will only free the #GdkColoramp
  *   object and not call XFreeColormap()
+ *
+ * Since: 2.2
  **/
 GdkColormap *
 gdk_x11_colormap_foreign_new (GdkVisual *visual,
@@ -1297,7 +1307,7 @@ gdk_x11_colormap_foreign_new (GdkVisual *visual,
   screen = gdk_visual_get_screen (visual);
   
   if (xcolormap == DefaultColormap (GDK_SCREEN_XDISPLAY (screen),
-				    GDK_SCREEN_XNUMBER (screen)));
+				    GDK_SCREEN_XNUMBER (screen)))
     return g_object_ref (gdk_screen_get_system_colormap (screen));
 
   colormap = gdk_colormap_lookup (screen, xcolormap);
@@ -1311,7 +1321,6 @@ gdk_x11_colormap_foreign_new (GdkVisual *visual,
 
   private->screen = screen;
   private->xcolormap = xcolormap;
-  private->xdisplay = GDK_SCREEN_XDISPLAY (screen);
   private->private_val = FALSE;
 
   colormap->size = visual->colormap_size;
@@ -1362,7 +1371,7 @@ gdkx_colormap_get (Colormap xcolormap)
   GdkColormap *colormap;
 
   if (xcolormap == DefaultColormap (GDK_SCREEN_XDISPLAY (screen),
-				    GDK_SCREEN_XNUMBER (screen)));
+				    GDK_SCREEN_XNUMBER (screen)))
     return g_object_ref (gdk_screen_get_system_colormap (screen));
 
   colormap = gdk_colormap_lookup (screen, xcolormap);
@@ -1385,9 +1394,6 @@ gdk_colormap_match_color (GdkColormap *cmap,
   gint rdiff, gdiff, bdiff;
   gint i, index;
 
-  g_return_val_if_fail (cmap != NULL, 0);
-  g_return_val_if_fail (color != NULL, 0);
-
   colors = cmap->colors;
   max = 3 * (65536);
   index = -1;
@@ -1485,7 +1491,7 @@ gdk_x11_colormap_get_xdisplay (GdkColormap *colormap)
 
   private = GDK_COLORMAP_PRIVATE_DATA (colormap);
 
-  return private->xdisplay;
+  return GDK_SCREEN_XDISPLAY (private->screen);
 }
 
 /**
@@ -1518,6 +1524,8 @@ gdk_x11_colormap_get_xcolormap (GdkColormap *colormap)
  * Gets the screen for which this colormap was created.
  * 
  * Return value: the screen for which this colormap was created.
+ *
+ * Since: 2.2
  **/
 GdkScreen *
 gdk_colormap_get_screen (GdkColormap *cmap)
@@ -1526,3 +1534,6 @@ gdk_colormap_get_screen (GdkColormap *cmap)
 
   return  GDK_COLORMAP_PRIVATE_DATA (cmap)->screen;
 }
+
+#define __GDK_COLOR_X11_C__
+#include "gdkaliasdef.c"
