@@ -1,6 +1,6 @@
 /* gtktoolitem.c
  *
- * Copyright (C) 2002 Anders Carlsson <andersca@codefactory.se>
+ * Copyright (C) 2002 Anders Carlsson <andersca@gnome.org>
  * Copyright (C) 2002 James Henstridge <james@daa.com.au>
  * Copyright (C) 2003 Soeren Sandmann <sandmann@daimi.au.dk>
  *
@@ -20,12 +20,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "gtktoolitem.h"
 #include "gtkmarshalers.h"
 #include "gtktoolbar.h"
 #include "gtkseparatormenuitem.h"
 #include "gtkintl.h"
 #include "gtkmain.h"
+#include "gtkalias.h"
+#include "gtkprivate.h"
 
 #include <string.h>
 
@@ -76,6 +79,8 @@ static void gtk_tool_item_get_property (GObject         *object,
 					guint            prop_id,
 					GValue          *value,
 					GParamSpec      *pspec);
+static void gtk_tool_item_property_notify (GObject      *object,
+					   GParamSpec   *pspec);
 static void gtk_tool_item_realize       (GtkWidget      *widget);
 static void gtk_tool_item_unrealize     (GtkWidget      *widget);
 static void gtk_tool_item_map           (GtkWidget      *widget);
@@ -136,6 +141,7 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
   object_class->set_property = gtk_tool_item_set_property;
   object_class->get_property = gtk_tool_item_get_property;
   object_class->finalize = gtk_tool_item_finalize;
+  object_class->notify = gtk_tool_item_property_notify;
 
   widget_class->realize       = gtk_tool_item_realize;
   widget_class->unrealize     = gtk_tool_item_unrealize;
@@ -150,32 +156,33 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
   
   g_object_class_install_property (object_class,
 				   PROP_VISIBLE_HORIZONTAL,
-				   g_param_spec_boolean ("visible_horizontal",
-							 _("Visible when horizontal"),
-							 _("Whether the toolbar item is visible when the toolbar is in a horizontal orientation."),
+				   g_param_spec_boolean ("visible-horizontal",
+							 P_("Visible when horizontal"),
+							 P_("Whether the toolbar item is visible when the toolbar is in a horizontal orientation."),
 							 TRUE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
   g_object_class_install_property (object_class,
 				   PROP_VISIBLE_VERTICAL,
-				   g_param_spec_boolean ("visible_vertical",
-							 _("Visible when vertical"),
-							 _("Whether the toolbar item is visible when the toolbar is in a vertical orientation."),
+				   g_param_spec_boolean ("visible-vertical",
+							 P_("Visible when vertical"),
+							 P_("Whether the toolbar item is visible when the toolbar is in a vertical orientation."),
 							 TRUE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
   g_object_class_install_property (object_class,
  				   PROP_IS_IMPORTANT,
- 				   g_param_spec_boolean ("is_important",
- 							 _("Is important"),
- 							 _("Whether the toolbar item is considered important. When TRUE, toolbar buttons show text in GTK_TOOLBAR_BOTH_HORIZ mode"),
+ 				   g_param_spec_boolean ("is-important",
+ 							 P_("Is important"),
+ 							 P_("Whether the toolbar item is considered important. When TRUE, toolbar buttons show text in GTK_TOOLBAR_BOTH_HORIZ mode"),
  							 FALSE,
- 							 G_PARAM_READWRITE));
+ 							 GTK_PARAM_READWRITE));
 
 /**
  * GtkToolItem::create-menu-proxy:
  * @toolitem: the object the signal was emitted on
  *
- * This signal is emitted when the toolbar is displaying an overflow menu.
- * In response the tool item should either 
+ * This signal is emitted when the toolbar needs information from @tool_item
+ * about whether the item should appear in the toolbar overflow menu. In
+ * response the tool item should either
  * <itemizedlist>
  * <listitem> call gtk_tool_item_set_proxy_menu_item() with a %NULL
  * pointer and return %TRUE to indicate that the item should not appear
@@ -190,7 +197,12 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
  * installs a menu item.
  * </listitem>
  * </itemizedlist>
- * 
+ *
+ * The toolbar may cache the result of this signal. When the tool item changes
+ * how it will respond to this signal it must call gtk_tool_item_rebuild_menu()
+ * to invalidate the cache and ensure that the toolbar rebuilds its overflow
+ * menu.
+ *
  * Return value: %TRUE if the signal was handled, %FALSE if not
  **/
   toolitem_signals[CREATE_MENU_PROXY] =
@@ -272,6 +284,9 @@ gtk_tool_item_finalize (GObject *object)
 {
   GtkToolItem *item = GTK_TOOL_ITEM (object);
 
+  if (item->priv->menu_item_id)
+    g_free (item->priv->menu_item_id);
+  
   if (item->priv->menu_item)
     g_object_unref (item->priv->menu_item);
   
@@ -300,7 +315,7 @@ gtk_tool_item_set_property (GObject      *object,
       gtk_tool_item_set_visible_horizontal (toolitem, g_value_get_boolean (value));
       break;
     case PROP_VISIBLE_VERTICAL:
-      gtk_tool_item_set_visible_horizontal (toolitem, g_value_get_boolean (value));
+      gtk_tool_item_set_visible_vertical (toolitem, g_value_get_boolean (value));
       break;
     case PROP_IS_IMPORTANT:
       gtk_tool_item_set_is_important (toolitem, g_value_get_boolean (value));
@@ -334,6 +349,17 @@ gtk_tool_item_get_property (GObject    *object,
     }
 }
 
+static void
+gtk_tool_item_property_notify (GObject    *object,
+			       GParamSpec *pspec)
+{
+  GtkToolItem *tool_item = GTK_TOOL_ITEM (object);
+
+  if (tool_item->priv->menu_item && strcmp (pspec->name, "sensitive") == 0)
+    gtk_widget_set_sensitive (tool_item->priv->menu_item,
+			      GTK_WIDGET_SENSITIVE (tool_item));
+}
+
 static void
 create_drag_window (GtkToolItem *toolitem)
 {
@@ -762,7 +788,7 @@ gtk_tool_item_set_is_important (GtkToolItem *tool_item, gboolean is_important)
 
       gtk_widget_queue_resize (GTK_WIDGET (tool_item));
 
-      g_object_notify (G_OBJECT (tool_item), "is_important");
+      g_object_notify (G_OBJECT (tool_item), "is-important");
     }
 }
 
@@ -889,7 +915,7 @@ gtk_tool_item_set_visible_horizontal (GtkToolItem *toolitem,
     {
       toolitem->priv->visible_horizontal = visible_horizontal;
 
-      g_object_notify (G_OBJECT (toolitem), "visible_horizontal");
+      g_object_notify (G_OBJECT (toolitem), "visible-horizontal");
 
       gtk_widget_queue_resize (GTK_WIDGET (toolitem));
     }
@@ -940,7 +966,7 @@ gtk_tool_item_set_visible_vertical (GtkToolItem *toolitem,
     {
       toolitem->priv->visible_vertical = visible_vertical;
 
-      g_object_notify (G_OBJECT (toolitem), "visible_vertical");
+      g_object_notify (G_OBJECT (toolitem), "visible-vertical");
 
       gtk_widget_queue_resize (GTK_WIDGET (toolitem));
     }
@@ -985,7 +1011,8 @@ gtk_tool_item_retrieve_proxy_menu_item (GtkToolItem *tool_item)
   
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), NULL);
 
-  g_signal_emit (tool_item, toolitem_signals[CREATE_MENU_PROXY], 0, &retval);
+  g_signal_emit (tool_item, toolitem_signals[CREATE_MENU_PROXY], 0,
+		 &retval);
   
   return tool_item->priv->menu_item;
 }
@@ -1021,6 +1048,34 @@ gtk_tool_item_get_proxy_menu_item (GtkToolItem *tool_item,
   return NULL;
 }
 
+/**
+ * gtk_tool_item_rebuild_menu()
+ * @tool_item: a #GtkToolItem
+ * 
+ * Calling this function signals to the toolbar that the
+ * overflow menu item for @tool_item has changed. If the
+ * overflow menu is visible when this function it called,
+ * the menu will be rebuilt.
+ *
+ * The function must be called when the tool item
+ * changes what it will do in response to the "create_menu_proxy"
+ * signal.
+ * 
+ * Since: 2.6
+ **/
+void
+gtk_tool_item_rebuild_menu (GtkToolItem *tool_item)
+{
+  GtkWidget *widget;
+  
+  g_return_if_fail (GTK_IS_TOOL_ITEM (tool_item));
+
+  widget = GTK_WIDGET (tool_item);
+  
+  if (widget->parent && GTK_IS_TOOLBAR (widget->parent))
+    _gtk_toolbar_rebuild_menu (GTK_TOOLBAR (widget->parent));
+}
+
 /**
  * gtk_tool_item_set_proxy_menu_item:
  * @tool_item: a #GtkToolItem:
@@ -1050,12 +1105,15 @@ gtk_tool_item_set_proxy_menu_item (GtkToolItem *tool_item,
   if (tool_item->priv->menu_item != menu_item)
     {
       if (tool_item->priv->menu_item)
-	g_object_unref (G_OBJECT (tool_item->priv->menu_item));
+	g_object_unref (tool_item->priv->menu_item);
       
       if (menu_item)
 	{
 	  g_object_ref (menu_item);
 	  gtk_object_sink (GTK_OBJECT (menu_item));
+
+	  gtk_widget_set_sensitive (menu_item,
+				    GTK_WIDGET_SENSITIVE (tool_item));
 	}
       
       tool_item->priv->menu_item = menu_item;
@@ -1079,6 +1137,11 @@ _gtk_tool_item_toolbar_reconfigured (GtkToolItem *tool_item)
 
   g_signal_emit (tool_item, toolitem_signals[TOOLBAR_RECONFIGURED], 0);
   
+  if (tool_item->priv->drag_window)
+    gdk_window_raise (tool_item->priv->drag_window);
+
   gtk_widget_queue_resize (GTK_WIDGET (tool_item));
 }
 
+#define __GTK_TOOL_ITEM_C__
+#include "gtkaliasdef.c"
